@@ -1,14 +1,18 @@
+import datetime
+import logging
 import os
 import sys
 from concurrent.futures import ThreadPoolExecutor
-import datetime
+from pathlib import Path
 from typing import List, Tuple, Union
+
+import requests
 import typer
+
 from chris.client import ChrisClient
 from chris.cube.files import DownloadableFile
+from chris.cube.pagination import TooMuchPaginationException, MAX_REQUESTS, REQUESTS_ENV_VAR_NAME
 from chris.types import CUBEUrl
-import logging
-from pathlib import Path
 
 
 def upload(client: ChrisClient, files: List[Path], parent_folder='', upload_threads=4):
@@ -83,23 +87,36 @@ def download(client: ChrisClient, url: Union[str, CUBEUrl], destination: Path, t
         os.makedirs(target.parent, exist_ok=True)
         return target, remote_file
 
-    search = tuple(client.get_files(url))
-    with typer.progressbar(search, length=len(search), label='Getting information', file=sys.stderr) as progress:
+    files_to_download = _discover_files_to_download(client, url)
+
+    with typer.progressbar(files_to_download, length=len(files_to_download), label='Getting information', file=sys.stderr) as progress:
         to_download = frozenset(__calculate_target(remote_file) for remote_file in progress)
 
     with typer.progressbar(length=len(to_download), label='Downloading files', file=sys.stderr) as progress:
-        def download_file(t: Tuple[Path, DownloadableFile]) -> int:
-            """
-            Download file and move the progress bar
-            :param t: tuple
-            :return: downloaded file size
-            """
-            target, remote_file = t
-            remote_file.download(target)
-            progress.update(1)
-            return target.stat().st_size
-
         with ThreadPoolExecutor(max_workers=threads) as pool:
+
+            def download_file(t: Tuple[Path, DownloadableFile]) -> int:
+                """
+                Download file and move the progress bar
+                :return: downloaded file size
+                """
+                target, remote_file = t
+                try:
+                    remote_file.download(target)
+                except requests.exceptions.RequestException as e:
+                    typer.secho(f'Failed to download {remote_file.file_resource}: {str(e)}',
+                                fg=typer.colors.RED, err=True)
+                    if sys.version_info.minor < 10:
+                        typer.secho(
+                            'This is probably a bug in Python itself. '
+                            'See https://github.com/FNNDSC/caw/issues/12',
+                            fg=typer.colors.YELLOW, err=True
+                        )
+                    pool.shutdown(cancel_futures=True)  # fail fast
+                    raise typer.Abort()
+                progress.update(1)
+                return target.stat().st_size
+
             sizes = pool.map(download_file, to_download)
 
     total_size = sum(sizes)
@@ -110,3 +127,26 @@ def download(client: ChrisClient, url: Union[str, CUBEUrl], destination: Path, t
     else:
         size = f'{total_size / 1e9:.4f} GB'
     typer.secho(size, fg=typer.colors.GREEN, err=True)
+
+
+def _discover_files_to_download(client: ChrisClient, url: CUBEUrl) -> Tuple[DownloadableFile, ...]:
+    typer.echo('Discovering files... ', nl=False)
+    total = 0
+
+    def report_discovered_file(f: DownloadableFile) -> DownloadableFile:
+        nonlocal total
+        total += 1
+        typer.echo(f'\rDiscovering files... {total}', nl=False)
+        return f
+
+    try:
+        search = tuple(report_discovered_file(f) for f in client.get_files(url))
+    except TooMuchPaginationException:
+        typer.echo(
+            f'Number of paginated requests exceeded {MAX_REQUESTS}.'
+            f"If you're trying to download many files, you can increase the limit:"
+            f'\n\n\t {" ".join([ "env", f"{REQUESTS_ENV_VAR_NAME}={MAX_REQUESTS + 99900}"] + sys.argv)}\n'
+        )
+        raise typer.Abort()
+    typer.echo(f'\rFound {total} files to download.')
+    return search
